﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using DoubleDeckerbus.Generic;
using DoubleDeckerbus.Drawing;
using DoubleDeckerbus.Move_Strategy;


namespace DoubleDeckerbus
{
    public partial class FormDoubleDeckerbusCollection : Form
    {
        private readonly BusesGenericStorage _storage;

        public FormDoubleDeckerbusCollection()
        {
            InitializeComponent();
            _storage = new BusesGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
        }
        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;
            listBoxStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i]);
            }
            if (listBoxStorages.Items.Count > 0 && (index == -1 || index >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 && index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }

        }
        private void buttonAddStorage_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
        }
        private void listBoxObjects_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image = _storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowBus();
        }
        private void buttonDeleteStorage_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить объект {listBoxStorages.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(listBoxStorages.SelectedItem.ToString() ?? string.Empty);
                ReloadObjects();
            }
        }

        private void buttonAddBus_Click(object sender, EventArgs e)
        {
            
            var formBusConfig = new FormBusConfig();
            formBusConfig.AddEvent(AddBus);
            formBusConfig.Show();
        }

        private void AddBus(DrawingBus selectedBus)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                MessageBox.Show("Не выбран набор");
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            selectedBus.ChangeBordersPicture(pictureBoxCollection.Width, pictureBoxCollection.Height);
            if (obj + selectedBus != -1)
              {
               MessageBox.Show("Объект добавлен");
               pictureBoxCollection.Image = obj.ShowBus();
              }
            else
               {
               MessageBox.Show("Не удалось добавить объект");
               }
        }

        private void buttonDeleteBus_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = 0;
            try
            {
                pos = Convert.ToInt32(maskedTextBoxNumber.Text);
            }
            catch
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (obj - pos)                                                                                                           
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = obj.ShowBus();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        private void buttonUpdate_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowBus();
        }
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (_storage.SaveData(saveFileDialog.FileName))
                {
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Не сохранилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (_storage.LoadData(openFileDialog.FileName))
                {
                    ReloadObjects();
                    var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
                    pictureBoxCollection.Image = obj.ShowBus();
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Не загрузилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
