﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DoubleDeckerbus.Drawing;
using DoubleDeckerbus.Move_Strategy;

namespace DoubleDeckerbus.Generic
{
    internal class BusGenericCollection<T, U>
        where T : DrawingBus
        where U : IMoveableObject
    {
        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        private readonly int _placeSizeWidth = 200;
        private readonly int _placeSizeHeight = 120;
        private readonly SetGeneric<T> _collection;

        public BusGenericCollection(int picWidth, int picHeight)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;

            _collection = new SetGeneric<T>(width * height);
        }
        public static int operator +(BusGenericCollection<T, U> collect, T? obj)
        {
            if (obj != null)
            {
                return collect._collection.Insert(obj);
            }
            return -1;
        }
        public static bool operator -(BusGenericCollection<T, U> collect, int pos)
        {
            if (collect._collection.GetBus(pos) == null)
            {
                return false;
            }
            return collect?._collection.Remove(pos) ?? false;
        }

        public IEnumerable<T?> GetBus => _collection.GetBus();
        public U? GetU(int pos)
        {
            return (U?)_collection[pos]?.GetMoveableObject;
        }

        public Bitmap ShowBus()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawObjects(gr);
            return bmp;
        }

        private void DrawBackground(Graphics gr)
        {
            Pen pen = new(Color.Black, 3);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; ++i)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
                {
                    gr.DrawLine(pen, i * _placeSizeWidth, j * _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j * _placeSizeHeight);
                    gr.DrawLine(pen, i * _placeSizeWidth, 0, i * _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
                }
            }
        }

        private void DrawObjects(Graphics g)
        {
            int x = _pictureWidth / _placeSizeWidth - 1;
            int y = 0;

            foreach (var bus in _collection.GetBus())
            {
                if (bus != null)
                {
                    if (x < 0)
                    {
                        x = _pictureWidth / _placeSizeWidth - 1;
                        ++y;
                    }
                    bus.SetPosition(_placeSizeWidth * x, _placeSizeHeight * y);
                    bus.DrawTransport(g);
                    --x;
                }
            }
        }
    }
 }
    
